<?php

namespace EventPortal\Tests\Unit\EventDelegate\Handlers;

use EventPortal\Domain\Delegate\Models\DelegateFormStep;
use EventPortal\Domain\Event\Form\Models\Form;
use EventPortal\Domain\Event\Form\Models\FormStep;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Event\Models\EventDelegate;
use EventPortal\Domain\EventDelegate\Handlers\RegisterDelegate;
use EventPortal\Domain\Form\Payload;
use EventPortal\Domain\Util\Notification\SystemNotification;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\RefreshDatabase;
use Illuminate\Pipeline\Pipeline;
use Illuminate\Support\Facades\Notification;

class RegisterDelegateTest extends TestCase
{
    use RefreshDatabase;

    /** @test **/
    public function can_transition_event_delegate_from_invited_to_registering(): void
    {
        $event = factory(Event::class)->create();
        $form = factory(Form::class)->create(['event_id' => $event->id]);
        $step1 = factory(FormStep::class)->create(['form_id' => $form->id]);
        factory(FormStep::class)->create(['form_id' => $form->id]);
        $eventDelegate = factory(EventDelegate::class)->state('invited')->create(['event_id' => $event->id]);
        factory(DelegateFormStep::class)
            ->create(['delegate_id' => $eventDelegate->delegate->id, 'form_step_id' => $step1->id]);

        $payload = Payload::make()
            ->withContext(['eventDelegate' => $eventDelegate, 'form' => $form, 'event' => $event]);

        app(Pipeline::class)
            ->send($payload)
            ->through([new RegisterDelegate])
            ->thenReturn();

        $this->assertEquals('registering', $eventDelegate->refresh()->status);
    }

    /** @test **/
    public function can_transition_event_delegate_from_invited_to_registered(): void
    {
        Notification::fake();

        $event = factory(Event::class)->create();
        $form = factory(Form::class)->create(['event_id' => $event->id]);
        $step = factory(FormStep::class)->create(['form_id' => $form->id]);
        $eventDelegate = factory(EventDelegate::class)->state('invited')->create(['event_id' => $event->id]);
        factory(DelegateFormStep::class)
            ->create(['delegate_id' => $eventDelegate->delegate->id, 'form_step_id' => $step->id]);

        $payload = Payload::make()
            ->withContext(['eventDelegate' => $eventDelegate, 'form' => $form, 'event' => $event]);

        app(Pipeline::class)
            ->send($payload)
            ->through([new RegisterDelegate])
            ->thenReturn();

        $this->assertEquals('registered', $eventDelegate->refresh()->status);

        Notification::assertSentTo(
            $eventDelegate->delegate->user,
            SystemNotification::class,
            static function (SystemNotification $notification) {
                return $notification->subject === 'Registration confirmation';
            }
        );
    }

    /** @test **/
    public function can_transition_event_delegate_from_registering_to_registered(): void
    {
        Notification::fake();

        $event = factory(Event::class)->create();
        $form = factory(Form::class)->create(['event_id' => $event->id]);
        $step = factory(FormStep::class)->create(['form_id' => $form->id]);
        $eventDelegate = factory(EventDelegate::class)->state('registering')->create(['event_id' => $event->id]);
        factory(DelegateFormStep::class)
            ->create(['delegate_id' => $eventDelegate->delegate->id, 'form_step_id' => $step->id]);

        $payload = Payload::make()
            ->withContext(['eventDelegate' => $eventDelegate, 'form' => $form, 'event' => $event]);

        app(Pipeline::class)
            ->send($payload)
            ->through([new RegisterDelegate])
            ->thenReturn();

        $this->assertEquals('registered', $eventDelegate->refresh()->status);

        Notification::assertSentTo(
            $eventDelegate->delegate->user,
            SystemNotification::class,
            static function (SystemNotification $notification) {
                return $notification->subject === 'Registration confirmation';
            }
        );
    }
}
